import java.util.*;

/*
  Cette classe implemente un B-arbre particulier qui est l'arbre 2-3-4.
  Cependant implementer un autre type de B-arbre il suffit juste de changer 
  la valeur de degreMin.
*/

class Arbre234 {
    protected int degreMin = 2; //Cas particulier des arbres 2-3-4
    protected Noeud racine;
    
    class Noeud {
		boolean feuille; //true si le noeud est une feuille false sinon
		Vector clefs; //Tableau des clefs (Integer) classees par ordre croissant
		Vector objets; //Tableau des objets (Vector de Fiche) associes aux clefs
		Vector fils; //Tableau des fils (Noeud)
		
		public Noeud () {
			feuille = true;
			clefs = new Vector();
			objets = new Vector();
			fils = new Vector();
		}

		/* Permet de separer en 2 sous-fils, un fils plein
		   c'est a dire ayant 2*degreMin-1 clefs.
		   Ce fils se trouve a la position 'indice' dans le Vector 'fils'*/
		public void decouperFils (int indice) {
			Noeud noeudGauche = (Noeud) fils.elementAt(indice);
			Noeud noeudDroite = new Noeud();
			noeudDroite.feuille = noeudGauche.feuille;
			int i;
			for (i=0; i<degreMin-1; i++) {
				noeudDroite.clefs.addElement(noeudGauche.clefs.elementAt(i+degreMin));
				noeudDroite.objets.addElement(noeudGauche.objets.elementAt(i+degreMin));
			}
			if (!noeudGauche.feuille) {
				for (i=0; i<degreMin; i++) {
					noeudDroite.fils.addElement(noeudGauche.fils.elementAt(i+degreMin));
				}
				noeudGauche.fils.setSize(degreMin);
			}
			fils.insertElementAt(noeudDroite,indice+1);
			clefs.insertElementAt(noeudGauche.clefs.elementAt(degreMin-1),indice);
			objets.insertElementAt(noeudGauche.objets.elementAt(degreMin-1),indice);
			noeudGauche.clefs.setSize(degreMin-1);
			noeudGauche.objets.setSize(degreMin-1);
		}

		/* Ajoute une fiche supplementaire au noeud si celui-ci n'est pas plein
		   ce qui a ete verifie avant. Par ailleurs, on s'est aussi assure que 'fic.id'
		   n'est pas un num de client deja present dans l'arbre */
		public void addNotFull (Fiche fic) {
			int i=0;
			if (feuille) {
				for (i=0; i<clefs.size(); i++) {
					if (fic.id < ((Integer)clefs.elementAt(i)).intValue())
					break;
				}
				clefs.insertElementAt(new Integer(fic.id),i);
				Vector vect = new Vector();
				vect.addElement(fic);
				objets.insertElementAt(vect, i);
			}
			else {
				for (i=0; i<clefs.size(); i++) {
					if (fic.id < ((Integer)clefs.elementAt(i)).intValue())
					break;
				}
				if (((Noeud)fils.elementAt(i)).clefs.size() == 2*degreMin-1) {
					decouperFils(i);
					/* Apres avoir decoupe le ieme fils il faut savoir ou inserer 'fic'
					   ce peut etre dans le ieme ou le i+1eme fils. */
					if (fic.id > ((Integer)clefs.elementAt(i)).intValue()) 
						i++;
				}
				((Noeud)fils.elementAt(i)).addNotFull(fic);
			}
		}

		/* Enumere recursivement tous les elts de l'arbre en les stockant ds
		   le Vector vect passe en param */
		public void elements (Vector vect) {
			for (int i=0 ; i<clefs.size() ; i++) {
				if (!feuille) {
					((Noeud)fils.elementAt(i)).elements(vect);
				}
				vect.addAll((Vector)objets.elementAt(i));
			}
			if (!feuille)
				((Noeud)fils.elementAt(clefs.size())).elements(vect);
		}

		/* Supprime un elt du ss-arbre ayant comme racine ce noeud 
		   en conservant sa structure */
		public void remove (int clef) {
			/* Si la clef se trouve directement dans ce noeud, on la supprime ds cette 
			   methode, sinon on fait appel a la methode removeInChild */
			if (feuille) {
				// Si le noeud est une feuille on supprime la clef si on la trouve
				for (int i=0 ; i<clefs.size() ; i++) {
					if (((Integer)clefs.elementAt(i)).intValue() == clef) {
						clefs.removeElementAt(i);
						objets.removeElementAt(i);
						break;
					}
				}
			}
			else { // Le noeud est un noeud interne
				Noeud filsGauche;
				Noeud filsDroit;
				int taille;
				boolean trouve = false;
				for (int i=0 ; i<clefs.size() ; i++) {
					if (((Integer)clefs.elementAt(i)).intValue() == clef) {
						/* Si la clef se trouve dans ce noeud, on essaye de conserver
						   la taille de ce noeud et de diminuer celle de ses fils */
							filsGauche = (Noeud)fils.elementAt(i);
							filsDroit = (Noeud)fils.elementAt(i+1);
						if ((taille=filsGauche.clefs.size()) >= degreMin) {
							/* Si le fils qui precede la clef a au moins degreMin clefs
							   on lui en retire une pour remplacer la clef supprimee
							   dans son pere; */
							clefs.setElementAt(filsDroit.clefs.elementAt(taille-1),i);
							objets.setElementAt(filsDroit.objets.elementAt(taille-1),i);
							filsDroit.clefs.removeElementAt(taille-1);
							filsDroit.objets.removeElementAt(taille-1);
						}
						else if ((taille=filsDroit.clefs.size()) >= degreMin) {
							/* Si le fils qui suit la clef a au moins degreMin clefs
							   on lui en retire une pour remplacer la clef supprimee
							   dans son pere; */
							clefs.setElementAt(filsDroit.clefs.elementAt(0),i);
							objets.setElementAt(filsDroit.objets.elementAt(0),i);
							filsDroit.clefs.removeElementAt(0);
							filsDroit.objets.removeElementAt(0);
						}
						else {
							/* Si les 2 fils n'ont que degreMin-1, on les fusionne
							   et on supprime la clef dans leur pere. */
							for (int j=0 ; j<taille ; j++) {
								filsGauche.clefs.addElement(filsDroit.clefs.elementAt(j));
								filsGauche.objets.addElement(filsDroit.objets.elementAt(j));
							}
							clefs.removeElementAt(i);
							objets.removeElementAt(i);
							fils.removeElementAt(i+1);
						}
						trouve = true;
						break;
					}
				}
				if (!trouve) { // La clef est dans un sous-noeud de ce noeud
					for (int i=0 ; i<clefs.size() ; i++) {
						// On cherche dans quel fils se trouve la clef si elle existe
						if (((Integer)clefs.elementAt(i)).intValue() > clef) {
							removeInChild(i,clef);
							trouve = true;
							break;
						}
					}
					if (!trouve) {
						removeInChild(clefs.size(),clef);
					}
				}
			}
		}
		
		/* Si l'elt a supprimer n'est pas ds ce noeud, mais ds l'un de ses fils, il
		   faut prendre qq precaution avant d'invoquer la methode 'remove' sur ce 
		   ss-noeud pour conserver la structure de l'arbre */
		protected void removeInChild (int i, int clef) {
			Noeud child;
			Noeud filsTmp;
			Noeud filsTmpGauche;
			Noeud filsTmpDroit;
			int taille;
			int tailleTmp;
			boolean tousMin;
			child = (Noeud)fils.elementAt(i);
			if (child.clefs.size() > degreMin-1) {
				/* Ce fils a suffisament de clefs pour qu'on puisse invoquer sur lui la methode 
				   'remove' sans risquer de compromettre la structure de l'arbre */
				child.remove(clef);
			}
			else { 
				/* Le fils trouve ne possede que degreMin-1 clefs, il faut augmenter sa taille
				   avant d'invoquer sur lui la methode 'remove' qui va peut-etre lui
				   enlever une clef et compromettre la structure de l'arbre */
				/* On regarde si un de ses freres a plus de clefs que lui
				   et pourrait lui en donner une */
				tousMin = true;
				// Peut-etre ses freres de droite
				for (int j=i+1 ; j<degreMin ; j++) {
					filsTmp = (Noeud) fils.elementAt(j);
					if (filsTmp.clefs.size() > degreMin-1) {
						tousMin = false;
						// On retire une clef a un de ses freres pour lui donner
						while (j != i) {
							j--;
							filsTmpGauche = (Noeud) fils.elementAt(j);
							// Deplacement des clefs
							filsTmpGauche.clefs.addElement(clefs.elementAt(j));
							clefs.setElementAt(filsTmp.clefs.elementAt(0),j);
							filsTmp.clefs.removeElementAt(0);
							// Deplacement des valeurs associees aux clefs
							filsTmpGauche.objets.addElement(objets.elementAt(j));
							objets.setElementAt(filsTmp.objets.elementAt(0),j);
							filsTmp.objets.removeElementAt(0);
						}
						// On invoque la methode 'remove'
						child.remove(clef);
						break;
					}
				}
				// Sinon ses freres de gauche
				if (tousMin) {
					for (int j=i-1 ; j>=0 ; j--) {
						filsTmp = (Noeud) fils.elementAt(j);
						tailleTmp = filsTmp.clefs.size();
						if (tailleTmp > degreMin-1) {
							tousMin = false;
							// On prend une clef d'un de ses freres pour lui donner
							while (j != i) {
								j++;
								filsTmpDroit = (Noeud) fils.elementAt(j);
								// Deplacement des clefs
								filsTmpDroit.clefs.insertElementAt(clefs.elementAt(j),0);
								clefs.setElementAt(filsTmp.clefs.elementAt(tailleTmp-1),j);
								filsTmp.clefs.removeElementAt(tailleTmp-1);
								// Deplacement des valeurs associees aux clefs
								filsTmpDroit.objets.insertElementAt(objets.elementAt(j),0);
								objets.setElementAt(filsTmp.objets.elementAt(tailleTmp-1),j);
								filsTmp.objets.removeElementAt(tailleTmp-1);
							}
							// On invoque la methode 'remove'
							child.remove(clef);
							break;
						}
					}
				}
				// Tous ses freres ont degreMin-1 clefs
				if (tousMin) {
					// On fusionne 2 freres cote a cote en descendant aussi une clef du pere
					if (i == fils.size()-1) {
						i--;
						child = (Noeud)fils.elementAt(i);
					}
					filsTmp = (Noeud)fils.elementAt(i+1);
					child.clefs.addElement(clefs.elementAt(i));
					child.objets.addElement(objets.elementAt(i));
					for (int k=0 ; k<degreMin-1 ; k++) {
						child.clefs.addElement(filsTmp.clefs.elementAt(k));
						child.objets.addElement(filsTmp.objets.elementAt(k));
					}
					clefs.removeElementAt(i);
					objets.removeElementAt(i);
					fils.removeElementAt(i+1);
					// On invoque la methode 'remove'
					child.remove(clef);
				}
			}
		}
    } // Fin de la classe Noeud
    
    // Construit un arbre vode ie avec une racine sans clef
    public Arbre234 () {
		racine = new Noeud();
    }

    // Teste si l'arbre est vide    
    public boolean estVide () {
		return (racine.clefs.size() == 0);
    }
    
    // Recherche dans l'arbre la clef 'clef' et retourne un Vector de Fiche si la 
    // clef existe sinon null. 
    public Vector rechercher (int clef) {
		if (estVide()) 
			return null;
		return rechercherRec(racine, clef);
	}
		
	protected Vector rechercherRec (Noeud noeud, int clef) {
		int i = 0;
		int nbClefs = noeud.clefs.size();
		while (i<nbClefs && clef>((Integer)noeud.clefs.elementAt(i)).intValue()) {
			i++;
		}
		if (i<nbClefs && clef==((Integer)noeud.clefs.elementAt(i)).intValue()) {
			return (Vector) noeud.objets.elementAt(i);
		}
		if (noeud.feuille) 
			return null;
		else
			return rechercherRec ((Noeud)noeud.fils.elementAt(i), clef);
    }
    
    // Ajoute une Fiche dans l'arbre
    public void add (Fiche fic) {
		Vector vect = rechercher(fic.id);
		if (vect == null) { //Le num de client n'est pas deja present dans l'arbre
			int nbClefs = racine.clefs.size();
			/* Dans un B-arbre un noeud contient au plus 2*degreMin-1 clefsx
			   soit 2*degreMin fils. Si on rajoute un element a un noeud plein il faut
			   le ceinder en deux */
			if (nbClefs == 2*degreMin-1) {
				Noeud r = racine;
				racine = new Noeud();
				racine.feuille = false;
				racine.fils.addElement(r);
				racine.decouperFils(0);
				racine.addNotFull(fic);
			}
			else {
				racine.addNotFull(fic);
			}
		}
		else //Le num de client fic.id est deja present dans l'arbre
			vect.addElement(fic);
    }

    // Retourne sous forme de Vector toutes les fiches de l'arbre
    public Vector elements () {
		Vector vect = new Vector();
		if (!estVide()) 
			racine.elements(vect);
		return vect;
	}

	// Supprime une clef
	public void removeKey (int clef) {
		racine.remove(clef);
		if (racine.clefs.size() == 0) { /* La racine ne contient plus aucune clef, on peut 
										  diminuer la hauteur de l'arbre */
			if (racine.fils.size() != 0) {
				racine = (Noeud)racine.fils.elementAt(0);
			}
		}
    }
}