import javax.swing.*;
import javax.swing.table.*;
import javax.swing.filechooser.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;

/*******************************************************************************************
  Cette classe est la classe principale du projet. Elle met en place l'interface graphique
  et implemente les fonctions pour manipuler le catalogue qui peut etre un tableau (Vector),
  une table de hachage (Hashtable) ou un arbre 2-3-4 (Arbre234). Pour cela, elle s'appuie 
  sur toutes les autres classes du projet.
********************************************************************************************/

public class Interface extends JFrame implements ActionListener{

    private JTable table; //Objet permettant d'afficher les fiches du catalogue
    private JScrollPane scrollPane;
    private JTextField idField; //Champ pour entrer le num du client a chercher

    private final JFileChooser fileChooser; //Boite de dialogue pour l'ouverture/enregistrement
    private final FileFilter rFilter; //|2 filres pour parametrer 'fileChooser' suivant
    private final FileFilter wFilter; //|que celle-ci est utilisee en ouverture ou fermeture

    private boolean modif; //Permet de savoir si le catalogue client a ete modifie
    private boolean rechercheFaite; //Pour savoir si une recherche a ete faite
    private boolean cataloguePret; /*Permet de savoir si un catalogue a ete ouvert ou cree
							  	     pour pouvoir le modifier */

    private File curCatFile; //Stocke le nom du fichier contenant le catalogue
    private Hashtable miniCatalogue; /*Pour rechercher plus rapidement que dans 'catalogue'
								       directement si un client si trouve a partir du nom*/
    private Object catalogue; //Stocke le catalogue
    private int firstFreeId; //Premier num de client libre
    private int typeCatalogue; /*Indique sous quelle forme le catalogue est stocke:
								 VECTOR, TABLE OU ARBRE*/
    private Vector listeRecherche; /*Stocke les elts trouves pendant une recherche
									 Ces elts peuvent etre soit des Integer soit des Fiche */

    private final static int VECTOR = 1; //Tableau (Vector)
    private final static int TABLE = 2; //Table de hachage (Hashtable)
    private final static int ARBRE = 3; //Arbre 2-3-4 (Arbre234)

	private Interface () {} // Constructeur interdit

    // Met en place l'interface graphique
    public Interface(int type) throws Exception {
		// Nom de l'application
        super("Catalogue des clients");

		modif = false;
		cataloguePret = false;
		rechercheFaite = false;
		rFilter = new RFilter();
		wFilter = new WFilter();

		// Initialisations de la structure qui contiendra le catalogue
		switch (type) {
		case VECTOR:
			catalogue = new Vector();
			break;
		case TABLE:
			catalogue = new Hashtable();
			break;
		case ARBRE:
			catalogue = new Arbre234();
			break;
		default:
			throw new Exception("Parametre incorrecte pour construire l'interface");
		}
		typeCatalogue = type;
		firstFreeId = -1;

		// Initialisation de la boite d'ouverture et de sauvegarde
		fileChooser = new JFileChooser(".");
		fileChooser.addChoosableFileFilter(new RWFilter());
		fileChooser.removeChoosableFileFilter(fileChooser.getAcceptAllFileFilter());

		// Creation du tableau ou seront affichees les fiches clients.
		table = new JTable();
		table.setPreferredScrollableViewportSize(new Dimension(580, 150));
		// Le tableau est muni de barres de defilement
		scrollPane = new JScrollPane(table);

		// Creation des 4 boutons pour manipuler les fiches clients.
		JButton bModifier = new JButton("Modifier");
		bModifier.setActionCommand("modifier");
		bModifier.addActionListener(this);
		JButton bSupprimer = new JButton("Supprimer");
		bSupprimer.setActionCommand("supprimer");
		bSupprimer.addActionListener(this);
		JButton bNouveau = new JButton("Creer une nouvelle fiche");
		bNouveau.setActionCommand("nouveau");
		bNouveau.addActionListener(this);
		JButton bChercher = new JButton("Rechercher un id");
		bChercher.setActionCommand("chercher");
		bChercher.addActionListener(this);

		// Mise en place de ces elements
		Container contentPane = getContentPane();
		// Les boutons 'bNouveau' et 'bChercher' et 'idField'
		JPanel buttonPanel = new JPanel();
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(10,10,5,10));
		buttonPanel.setLayout(new BoxLayout(buttonPanel,BoxLayout.X_AXIS));
		buttonPanel.add(bChercher);
		buttonPanel.add(Box.createRigidArea(new Dimension(10,0)));
		buttonPanel.add(new JLabel("id: "));
		idField = new JTextField(4);
		idField.setMaximumSize(new Dimension(10,30));
		buttonPanel.add(idField);
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(bNouveau);
        contentPane.add(buttonPanel, BorderLayout.NORTH);
		// La liste 'table' et les boutons 'bModifier' et 'bSupprimer'
		JPanel resultPanel = new JPanel();
		resultPanel.setBorder(BorderFactory.createCompoundBorder( 
							  BorderFactory.createTitledBorder(
									"Resultat de la recherche"),
							  BorderFactory.createEmptyBorder(5,5,5,5)));
		resultPanel.setLayout(new BorderLayout());
        resultPanel.add(scrollPane, BorderLayout.NORTH);
		JPanel optionPanel = new JPanel();
		optionPanel.setLayout(new BoxLayout(optionPanel,BoxLayout.X_AXIS));
		optionPanel.add(bModifier);
		optionPanel.add(Box.createRigidArea(new Dimension(10,0)));
		optionPanel.add(bSupprimer);
        resultPanel.add(optionPanel, BorderLayout.SOUTH);
        resultPanel.add(new Label("Operations sur les fiches selectionnees :"),
					  BorderLayout.WEST);
        contentPane.add(resultPanel, BorderLayout.CENTER);

		// Creation d'un menu
        JMenuBar menuBar;
        JMenu menu, submenu;
        JMenuItem menuItem;
        menuBar = new JMenuBar();
        setJMenuBar(menuBar);
        menu = new JMenu("Fichier");
        menuItem = new JMenuItem("Nouveau catalogue");
		menuItem.setActionCommand("nouveau catalogue");
		menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Ouvrir catalogue");
		menuItem.setActionCommand("ouvrir");
		menuItem.addActionListener(this);
        menu.add(menuItem);
        menu.addSeparator();
        menuItem = new JMenuItem("Sauvegarder");
		menuItem.setActionCommand("sauvegarder");
		menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Sauvegarder sous ...");
		menuItem.setActionCommand("sauvegarder sous");
		menuItem.addActionListener(this);
        menu.add(menuItem);
        menu.addSeparator();
        menuItem = new JMenuItem("Fermer");
		menuItem.setActionCommand("fermer");
		menuItem.addActionListener(this);
        menu.add(menuItem);
        menuBar.add(menu);
        menu = new JMenu("Outils");
        menuItem = new JMenuItem("Creer une nouvelle fiche");
		menuItem.setActionCommand("nouveau");
		menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Rechercher un id");
		menuItem.setActionCommand("chercher");
		menuItem.addActionListener(this);
        menu.add(menuItem);
        menu.addSeparator();
        menuItem = new JMenuItem("Afficher catalogue entier");
		menuItem.setActionCommand("afficherCat");
		menuItem.addActionListener(this);
        menu.add(menuItem);
        menuBar.add(menu);

		// Permet de fermer la fenetre sans demande de confirmation
		addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			});
    }

    // Gestion des actions declenchees par les boutons et le menu
    public void actionPerformed (ActionEvent e) {
		if (e.getActionCommand().equals("afficherCat")) {
			if (cataloguePret) {
				afficherCat();
			}
			else {
				String txt = "Operation interdite.\n" +
					"Ouvrez un catalogue pour en afficher le contenu.";
				JOptionPane.showMessageDialog(this,
											  txt,
											  "Operation interdite",
											  JOptionPane.ERROR_MESSAGE);
			}
		}
		else if (e.getActionCommand().equals("modifier")) {
			if (rechercheFaite) {
				modifier();
			}
			else {
				String txt = "Operation interdite.\n" +
					"Faite une recherche dans le catalogue pour pouvoir modifier des fiches.";
				JOptionPane.showMessageDialog(this,
											  txt,
											  "Operation interdite",
											  JOptionPane.ERROR_MESSAGE);
			}
		}
		else if (e.getActionCommand().equals("supprimer")) {
			if (rechercheFaite) {
				supprimer();
			}
			else {
				String txt = "Operation interdite.\n" +
					"Faite une recherche dans le catalogue pour pouvoir supprimer des fiches.";
				JOptionPane.showMessageDialog(this,
											  txt,
											  "Operation interdite",
											  JOptionPane.ERROR_MESSAGE);
			}
		}
		else if (e.getActionCommand().equals("nouveau")) {
			if (cataloguePret) {
				ajouter();
			}
			else {
				String txt = "Operation interdite.\n" +
					"Ouvrez un catalogue avant de creer une nouvelle fiche.";
				JOptionPane.showMessageDialog(this,
											  txt,
											  "Operation interdite",
											  JOptionPane.ERROR_MESSAGE);
			}
		}
		else if (e.getActionCommand().equals("chercher")) {
			if (cataloguePret) {
				rechercher();
			}
			else {
				String txt = "Operation interdite.\n" +
					"Ouvrez un catalogue pour faire une recherche.";
				JOptionPane.showMessageDialog(this,
											  txt,
											  "Operation interdite",
											  JOptionPane.ERROR_MESSAGE);
			}
		}
		else if (e.getActionCommand().equals("fermer")) {
			fermer();
		}
		else if (e.getActionCommand().equals("sauvegarder sous")) {
			sauvegarderSous();
		}
		else if (e.getActionCommand().equals("sauvegarder")) {
			sauvegarder();
		}
		else if (e.getActionCommand().equals("ouvrir")) {
			ouvrir();
		}
		else if (e.getActionCommand().equals("nouveau catalogue")) {
			newCat();
		}
    }
    
    // Specifie la largeur de chq colonne de la table affichant les fiches
    protected void initColumnSize () {
		table.getColumnModel().getColumn(0).setPreferredWidth(15);
		table.getColumnModel().getColumn(1).setPreferredWidth(25);
		table.getColumnModel().getColumn(2).setPreferredWidth(55);
		table.getColumnModel().getColumn(3).setPreferredWidth(100);
		table.getColumnModel().getColumn(4).setPreferredWidth(135);
		table.getColumnModel().getColumn(5).setPreferredWidth(80);
		table.getColumnModel().getColumn(6).setPreferredWidth(100);
		table.getColumnModel().getColumn(7).setPreferredWidth(70);
    }

    // Cree un nouveau catalogue
    protected void newCat () {
		boolean ok = false;
		int n;
		if (modif) { //Un catalogue client est deja ouvert et non sauvegarde
			// Affiche une boite de dialogue pour sauvegarder le 1er catalogue
			Object[] options = {"Sauvegarder","Ecraser","Annuler"};
			String message = "Un catalogue clients est deja ouvert et a ete modifie.\n"
				+ "Voulez-vous le sauvegarder avant d'en creer un autre ?";
			String titre = "Voulez-vous sauvegarder le catalogue ouvert ?";
			n = JOptionPane.showOptionDialog(this,
											 message,
											 titre,
											 JOptionPane.YES_NO_CANCEL_OPTION,
											 JOptionPane.QUESTION_MESSAGE,
											 null,
											 options,
											 options[0]);
			if (n == JOptionPane.YES_OPTION) { //Demande de sauvegarde av creation
				sauvegarder();
				ok = true;
			}
			else if (n == JOptionPane.NO_OPTION) { //Ouvrir sans sauvegarder
				ok = true;
			}
		}
		if (ok || !modif) {
			//L'utilisateur n'a ni ferme la boite de dialogue, ni choisit 'annuler'
			if (cataloguePret) {
				table = new JTable();
				scrollPane.setViewportView(table);
				rechercheFaite = false;
			}
			firstFreeId = 1; //Le 1er num attribu a un client est 1
			switch (typeCatalogue) {
			case VECTOR:
				catalogue = new Vector();
				break;
			case TABLE:
				catalogue = new Hashtable();
				break;
			case ARBRE:
				catalogue = new Arbre234();
				break;
			}
			miniCatalogue = new Hashtable();
			cataloguePret = true;
			modif = false;
			curCatFile = null;
		}
    }

    // Ouvre un catalogue a partir d'un fichier existant
    protected void ouvrir () {
		boolean open = false;
		int n;
		if (modif) { //Un catalogue client est deja ouvert et non sauvegarde
			// Affiche une boite de dialogue pour sauvegarder le 1er catalogue
			Object[] options = {"Sauvegarder","Ecraser","Annuler"};
			String message = "Un catalogue clients est deja ouvert et a ete modifie.\n"
				+ "Voulez-vous le sauvegarder avant d'en ouvrir un autre ?";
			String titre = "Voulez-vous sauvegarder le catalogue ouvert ?";
			n = JOptionPane.showOptionDialog(this,
											 message,
											 titre,
											 JOptionPane.YES_NO_CANCEL_OPTION,
											 JOptionPane.QUESTION_MESSAGE,
											 null,
											 options,
											 options[0]);
			if (n == JOptionPane.YES_OPTION) { //Demande de sauvegarde av ouverture
				sauvegarder();
				open = true;
			}
			else if (n == JOptionPane.NO_OPTION) { //Ouvrir sans sauvegarder
				open = true;
			}
		}
		if (open || !modif) {
			//L'utilisateur n'a ni ferme la boite de dialogue, ni choisit 'annuler'
			fileChooser.addChoosableFileFilter(rFilter);
			fileChooser.removeChoosableFileFilter(wFilter);
			int option = fileChooser.showOpenDialog(this);
			if (option == JFileChooser.APPROVE_OPTION) {
				if (cataloguePret) {
					cataloguePret = false;
					table = new JTable();
					scrollPane.setViewportView(table);
					rechercheFaite = false;
				}
				miniCatalogue = new Hashtable();
				curCatFile = fileChooser.getSelectedFile();
				Lecture lect = new Lecture(curCatFile);
				try {
					switch (typeCatalogue) {
					case VECTOR:
						catalogue = new Vector();
						firstFreeId = lect.read((Vector)catalogue, miniCatalogue);
						cataloguePret = true;
						break;
					case TABLE:
						catalogue = new Hashtable();
						firstFreeId = lect.read((Hashtable)catalogue, miniCatalogue);
						cataloguePret = true;
						break;
					case ARBRE:
						catalogue = new Arbre234();
						firstFreeId = lect.read((Arbre234)catalogue, miniCatalogue);
						cataloguePret = true;
						break;
					}
				}
				catch (CorruptedFileException e) {
					String txt = "Le format du fichier texte est errone.\n" +
					"Impossible de retrouver les informations de ce catalogue.\n" +
					"Veuillez ouvrir un autre fichier.\n";
					JOptionPane.showMessageDialog(this,
												  txt,
												  "Erreur a l'ouverture",
												  JOptionPane.ERROR_MESSAGE);
				}
				catch (IOException e) {
					String txt = "Erreur de lecture dans le fichier\n" + curCatFile +
					"Veuillez ouvrir un autre fichier.\n";
					JOptionPane.showMessageDialog(this,
												  txt,
												  "Erreur a l'ouverture",
												  JOptionPane.ERROR_MESSAGE);
				}
				finally {
					try {
						modif = false;
						lect.ferme();
					}
					catch (Exception e) {}
				}
			}
		}
    }		

    // Ajoute une nouvelle fiche au catalogue
    protected void ajouter () {
		Fiche fic = new Fiche();
		// Affichage d'une boite de dialogue pour remplir cette nouvelle fiche
		NewFicheDialog ficheDialog = new NewFicheDialog(fic,this,true);
		ficheDialog.pack();
		ficheDialog.show();
		if (fic.nom != null) { //L'utilisateur n'a pas choisit 'Annuler'
			// Remise a zero de la table si une recherche venait d'etre faite
			table = new JTable();
			scrollPane.setViewportView(table);
			rechercheFaite = false;
			// Attribution d'un id a la fiche
			Vector vect;
			Fiche tmp;
			boolean exist = false;
			if ((vect=(Vector) miniCatalogue.get(fic.nom)) != null) {
				for (int i=0 ; (i<vect.size()) && !exist ; i++) {
					tmp = (Fiche) vect.elementAt(i);
					if (tmp.prenom.compareTo(fic.prenom) == 0) { 
						//Le client est deja dans le catalogue
						exist = true;
						fic.id = tmp.id;
					}
				}
			}
			if (!exist) { //Le client est mis pour la 1ere fois dans le catalogue
				fic.id = firstFreeId;
				vect = new Vector();
				vect.addElement(fic);
				miniCatalogue.put(fic.nom,vect);
				//Recherche d'un nouveau 1er id libre
				//Celui-ci vaut au moins fic.id+1
				firstFreeId++;
				while (!isFreeId(firstFreeId)) {
					firstFreeId++;
				}
			}
			// Insertion de la fiche dans le catalogue
			switch (typeCatalogue) {
			case VECTOR:
				((Vector)catalogue).addElement(fic);
				break;
			case TABLE:
				if (!exist) {
					vect = new Vector();
					((Hashtable)catalogue).put(new Integer(fic.id),vect);
				}
				else {
					vect = (Vector)((Hashtable)catalogue).get(new Integer(fic.id));
				}
				vect.addElement(fic);
				break;
			case ARBRE:
				((Arbre234)catalogue).add(fic);
				break;
			}
			modif = true;
			// Informe a l'utilisateur que tout s'est bien deroule
			String txt = "La fiche :\n" + 
				"----------------------\n" +
				fic + "\n" +
				"----------------------\n" +
				"a ete ajoutee avec succes a votre catalogue.";
			JOptionPane.showMessageDialog(this,
										  txt,
										  "Nouvelle fiche",
										  JOptionPane.INFORMATION_MESSAGE);
		}
    }	

    // Teste si un num de client n'est plus utilise
    protected boolean isFreeId (int id) {
		boolean free = true;
		Fiche fic;
		switch (typeCatalogue) {
		case VECTOR:
			for (int i=0 ; i<((Vector)catalogue).size() && free ; i++) {
				fic = (Fiche) ((Vector)catalogue).elementAt(i);
				if (fic.id == id)
					return false;
			}
			return true;
		case TABLE:
			return ((Hashtable)catalogue).get(new Integer(id)) == null;
		case ARBRE:
			return ((Arbre234)catalogue).rechercher(id) == null;
		}
		return true;
    }

    // Recherche ds le catalogue ttes les fiches ayant comme num de client
    // celui qui se trouve ds 'idField'
    protected void rechercher () {
		// Si une recherche a deja ete faite, elle est annulee
		rechercheFaite = false;
		table = new JTable();
		scrollPane.setViewportView(table);
		listeRecherche = new Vector();
		try { // Verification du type d'entree presente ds le champ id
			// Ce champ ne doit pas etre vide
			if (idField.getText().length() == 0) {
				String txt = "Veuillez remplir le champ id";
				JOptionPane.showMessageDialog(this,
											  txt,
											  "Information manquante",
											  JOptionPane.ERROR_MESSAGE);
			}
			else {
				Fiche fic;
				Vector vect = null;
				int id = (new Integer(idField.getText())).intValue();
				switch (typeCatalogue) {
				case VECTOR:
					for (int i=0 ; i<((Vector)catalogue).size() ; i++) {
						fic = (Fiche)((Vector)catalogue).elementAt(i);
						if (fic.id == id) {
							listeRecherche.addElement(new Integer(i));
						}
					}   
					// Pour l'affichage si au moins 1 fiche a ete trouvee
					if (listeRecherche.size() != 0) { 
						vect = new Vector();
						for (int i=0 ; i<listeRecherche.size() ; i++) {
							fic = (Fiche)((Vector)catalogue).elementAt(
								 ((Integer)listeRecherche.elementAt(i)).intValue());
							vect.addElement(fic);
						}
					}
					break;
				case TABLE:
					listeRecherche = (Vector) ((Hashtable)catalogue).get(new Integer(id));
					vect = listeRecherche;
					break;
				case ARBRE:
					listeRecherche = ((Arbre234)catalogue).rechercher(id);
					vect = listeRecherche;
					break;
				}
				// Affichage
				if (vect != null) {
					rechercheFaite = true;
					afficherFiches(vect);
				}
				else {
					String txt = "Aucune fiche avec cet id n'a ete trouvee.";
					JOptionPane.showMessageDialog(this,
												  txt,
												  "Champ incorrecte",
												  JOptionPane.INFORMATION_MESSAGE);
				}
			}
		}
		catch (NumberFormatException e) {
			String txt = "Le champ id ne doit contenir que des entiers\n"
				+ "superieur ou egal a 1.";
			JOptionPane.showMessageDialog(this,
										  txt,
										  "Champ incorrecte",
										  JOptionPane.ERROR_MESSAGE);
		}
    }

    // Affiche a l'ecran toutes les fiches se trouvant ds 'vect' passe en param
    protected void afficherFiches (Vector vect) {
		Object[][] tab = new Object[vect.size()][8];
		Fiche fic;
		for (int i=0 ; i<vect.size() ; i++) {
			fic = (Fiche) vect.elementAt(i);
			tab[i][0] = new Boolean(false);
			tab[i][1] = new Integer(i);
			tab[i][2] = new Integer(fic.id);
			tab[i][3] = fic.nom;
			tab[i][4] = fic.prenom;
			String date = "" + 
				fic.date.getDate() + "/" + 
				(fic.date.getMonth()+1) + "/" + 
				(fic.date.getYear()+1900);
			tab[i][5] = date;
			tab[i][6] = fic.voiture;
			tab[i][7] = new Double(fic.prix);
		}
		Listing listing = new Listing (tab);
		table = new JTable(listing);
		initColumnSize();
		scrollPane.setViewportView(table);
    }

    // Affiche le contenu du catalogue en entier
    protected void afficherCat () {
		Enumeration enum;
		Vector vect = new Vector();
		switch (typeCatalogue) {
		case VECTOR:
			afficherFiches((Vector)catalogue);
			break;
		case TABLE:
			enum = ((Hashtable)catalogue).elements();
			while (enum.hasMoreElements()) {
				vect.addAll((Vector)enum.nextElement());
			}
			afficherFiches(vect);
			break;
		case ARBRE:
			vect = ((Arbre234)catalogue).elements();
			afficherFiches(vect);
			break;
		}
		rechercheFaite = false;
    }

    // Permet de modifier les fiches selectionnees ds 'table'
    protected void modifier () {
		boolean select = false;
		Fiche fic = new Fiche();
		int jour, mois, annee, j;
		String s;
		for (int i=0 ; i<table.getRowCount() ; i++) {
			// On cherche les fiches a modifier
			if (((Boolean)table.getValueAt(i,0)).booleanValue()) {
				select = true;
				//Constitution d'une fiche a partir des donnees selectionnes ds 'table'
				fic.id = ((Integer)table.getValueAt(i,2)).intValue();
				fic.nom = (String)table.getValueAt(i,3);
				fic.prenom = (String)table.getValueAt(i,4);
				fic.voiture = (String)table.getValueAt(i,6);
				fic.prix = ((Double)table.getValueAt(i,7)).doubleValue();
				s = (String)table.getValueAt(i,5);
				j = 1;
				while (s.charAt(j) != '/') {
					j++;
				}
				jour = (new Integer(s.substring(0,j))).intValue();
				s = s.substring(++j,s.length());
				j = 1;
				while (s.charAt(j) != '/') {
					j++;
				}
				mois = (new Integer(s.substring(0,j))).intValue();
				annee = (new Integer(s.substring(++j,s.length()))).intValue();
				fic.date = new Date(annee-1900, mois-1, jour, 0, 0, 0);
				// Affichage de cette fiche dans une boite de dialogue pour la modifier
				ModifFicheDialog dialog = new ModifFicheDialog(fic,this,true);
				dialog.pack();
				dialog.show();
				// Affichage des modif dans 'table'
				table.setValueAt(new Boolean(false),i,0);
				table.setValueAt(fic.voiture,i,6);
				table.setValueAt(new Double(fic.prix),i,7);
				s = "" + 
					fic.date.getDate() + "/" + 
					(fic.date.getMonth()+1) + "/" + 
					(fic.date.getYear()+1900);
				table.setValueAt(s,i,5);
				// Modification de la fiche dans le catalogue
				switch (typeCatalogue) {
				case VECTOR:
					j = ((Integer)listeRecherche.elementAt(i)).intValue();
					((Vector)catalogue).setElementAt(fic,j);
					break;
				case TABLE:
					listeRecherche.setElementAt(fic,i);
					break;
				case ARBRE:
					listeRecherche.setElementAt(fic,i);
					break;
				}
				modif = true;
			}
		}
		if (!select) { //aucune fiche n'a ete selectionnee
			String txt = "Aucune fiche selectionnee.\n"
				+ "Pour modifier une fiche, cochez la case correspondante a celle-ci.";
			JOptionPane.showMessageDialog(this,
										  txt,
										  "Manque d'information",
										  JOptionPane.WARNING_MESSAGE);
		}
    }

    // Permet de supprimer les fiches selectionnees ds 'table'
    protected void supprimer () {
		boolean select = false;
		for (int i=table.getRowCount()-1 ; i>=0 ; i--) {
			if (((Boolean)table.getValueAt(i,0)).booleanValue()) {
				select = true;
				// Suppression de la fiche ds la catalogue
				supprimerFiche(i);
				// Message de confirmation de la suppression
				String txt = "Supression effectuee avec succes.\n";
				JOptionPane.showMessageDialog(this,
											  txt,
											  "Fiche supprimee",
											  JOptionPane.INFORMATION_MESSAGE);
				// Affichage des modif ds 'table'
				Listing oldListing = (Listing)table.getModel();
				table = new JTable(oldListing.remove(i));
				initColumnSize();
				scrollPane.setViewportView(table);
			}
		}
		if (!select) { //aucune fiche n'a ete selectionnee
			String txt = "Aucune fiche selectionnee.\n"
				+ "Pour supprimer une fiche, cochez la case correspondante a celle-ci.";
			JOptionPane.showMessageDialog(this,
										  txt,
										  "Manque d'information",
										  JOptionPane.WARNING_MESSAGE);
		}
    }

    // Permet de supprimer une fiche du catalogue 
    protected void supprimerFiche (int indice) {
		int id = ((Integer)table.getValueAt(indice,2)).intValue();
		switch (typeCatalogue) {
		case VECTOR:
			// Suppression dans la table
			int j = ((Integer)listeRecherche.elementAt(indice)).intValue();
			((Vector)catalogue).removeElementAt(j);
			// Verification du 1er num de client libre
			if (id < firstFreeId) {
				if (isFreeId(id)) {
					firstFreeId = id;
				}
			}
			break;
		case TABLE:
			if (listeRecherche.size() == 1) {
				// Une seule fiche correspond a cet id, on peut supprimer la clef entiere
				Fiche fic = (Fiche)listeRecherche.elementAt(indice);
				((Hashtable)catalogue).remove(new Integer(fic.id));
				// Changement du 1er num de client si necessaire
				if (id < firstFreeId) {
					firstFreeId = id;
				}
			}
			else {
				listeRecherche.removeElementAt(indice);
			}
			break;
		case ARBRE:
			if (listeRecherche.size() == 1) {
				// Une seule fiche correspond a cet id, on peut supprimer la clef entiere
				Fiche fic = (Fiche)listeRecherche.elementAt(indice);
				((Arbre234)catalogue).removeKey(fic.id);
				// Changement du 1er num de client si necessaire
				if (id < firstFreeId) {
					firstFreeId = id;
				}
			}
			else {
				listeRecherche.removeElementAt(indice);
			}
			break;
		}
		modif = true;
    }

    // Sauvegarde du catalogue ds un autre fichier que 'curCatFile'
    protected void sauvegarderSous () {
		// Affichage de la boite de dialogue de sauvegarde
		fileChooser.addChoosableFileFilter(wFilter);
		fileChooser.removeChoosableFileFilter(rFilter);
		int option = fileChooser.showSaveDialog(this);
		if (option == JFileChooser.APPROVE_OPTION) {
			// Changement en consequence de 'curCatFile'
			curCatFile = fileChooser.getSelectedFile();
			// Sauvegarde ds le nouveau fichier courant
			sauvegarder();
		}   
    }

    // Sauvegarde du catalogue ds le fichier courant 'curCatFile'
    protected void sauvegarder () {
		if (curCatFile == null) { //Nouveau catalogue qui n'a encore jamais ete sauvegarde
			sauvegarderSous();
		}
		else {
			if (curCatFile.canWrite() || !curCatFile.exists()) {
				try {
					Ecriture ecrit = new Ecriture(curCatFile);
					Enumeration enum;
					Vector vect = new Vector();
					switch (typeCatalogue) {
					case VECTOR:
						ecrit.sauvegarde((Vector)catalogue,firstFreeId);
						break;
					case TABLE:
						enum = ((Hashtable)catalogue).elements();
						while (enum.hasMoreElements()) {
							vect.addAll((Vector)enum.nextElement());
						}
						ecrit.sauvegarde(vect,firstFreeId);
						break;
					case ARBRE:
						vect = ((Arbre234)catalogue).elements();
						ecrit.sauvegarde(vect,firstFreeId);
						break;
					}
					ecrit.ferme();
					modif = false;
					// On indique que la sauvegarde s'est bien passee
					String txt = "La sauvegarde s'est bien deroulee.";
					JOptionPane.showMessageDialog(this,
												  txt,
												  "Sauvegarde effectuee",
												  JOptionPane.INFORMATION_MESSAGE);
				}
				catch (IOException e) {
					String txt = "Sauvegarde impossible.\n"
						+ "Une erreur inconnue est survenue pendant la sauvegarde.\n"
						+ "Verifiez que vous dispose de suffisament d'espace disque.\n"
						+ "Rouvrez le fichier\n"
						+ curCatFile
						+ "pour vous assurer qu'il n'a ete corrompu.";
					JOptionPane.showMessageDialog(this,
												  txt,
												  "Sauvegarde impossible",
												  JOptionPane.ERROR_MESSAGE);
					// Reinit de l'application
					rechercheFaite = false;
					cataloguePret = false;
					modif = false;
					table = new JTable();
					scrollPane.setViewportView(table);
				}
			}
			else { //On n'a pas les droits en ecriture
			String txt = "Sauvegarde impossible.\n"
				+ "Vous n'avez pas le droit d'ecrire sur le fichier\n"
				+ curCatFile;
			JOptionPane.showMessageDialog(this,
										  txt,
										  "Sauvegarde impossible",
										  JOptionPane.ERROR_MESSAGE);
			}
		}
    }

    // Fermeture de l'application
    protected void fermer () {
		boolean close = false;
		if (modif) { // Le catalogue ouvert n'a pas ete sauvegarde
			// Affiche une boite de dialogue pour sauvegarder le catalogue ouvert
			Object[] options = {"Sauvegarder","Quitter","Annuler"};
			String message = "Le catalogue clients ouvert a ete modifie.\n"
				+ "Voulez-vous le sauvegarder avant de quitter ?";
			String titre = "Quitter ?";
			int n = JOptionPane.showOptionDialog(this,
												 message,
												 titre,
												 JOptionPane.YES_NO_CANCEL_OPTION,
												 JOptionPane.QUESTION_MESSAGE,
												 null,
												 options,
												 options[0]);
			if (n == JOptionPane.YES_OPTION) { //Demande de sauvegarde
				sauvegarder();
				close = true;
			}
			else if (n == JOptionPane.NO_OPTION) {
				close = true;
			}
		}
		if (close || !modif) {
			System.exit(0);
		}
    }			

    // Lancement de l'application
    public static void main(String[] args) throws Exception {
		// On choisit la structure du catalogue clients pour construire l'interface
		Object[] options = {"Vector"," Table de hachage ","Arbre 2-3-4"};
		String message = "Le catalogue clients peut etre manipule "
			+ "a partir de trois structures.\n"
			+ "Laquelle voulez-vous utiliser ?";
		String titre = "Structure du catalogue";
		int n = JOptionPane.showOptionDialog(null,
											 message,
											 titre,
											 JOptionPane.YES_NO_CANCEL_OPTION,
											 JOptionPane.QUESTION_MESSAGE,
											 null,
											 options,
											 options[0]);
		Interface frame = null;
		if (n == JOptionPane.YES_OPTION) { //Vector
			frame = new Interface(VECTOR);
		}
		else if (n == JOptionPane.NO_OPTION) { //Table de hachage
			frame = new Interface(TABLE);
		}
		else if (n == JOptionPane.CANCEL_OPTION) { //Arbre 2-3-4
			frame = new Interface(ARBRE);
		}
		else {
			System.exit(0);
		}
		// Affichage de l'interface
		frame.pack();
        frame.setVisible(true);
    }
}
