import java.io.*;
import java.util.*;
 
/************************************************************************
 Classe permettant de lire le fichier passe en parametre au constructeur
   pour remplir le catalogue 
*************************************************************************/

class Lecture {
    protected StreamTokenizer stoken;
    protected BufferedReader flot;
    
    // Constructeurs
    private Lecture () { // Oblige a passer le nom du fichier en param du constructeur
    }

    public Lecture (File nom) { // A partir d'un fichier
		try { // On traite directement les erreurs de lecture dans le constructeur
			FileInputStream  fic1;
			fic1 = new FileInputStream (nom); // Necessaire pour lire dans un fichier
			flot = new BufferedReader(new InputStreamReader(fic1)); /* Cre un acces en lecture
									    							   bufferisee sur ce fichier */
			stoken = new StreamTokenizer(flot); // Facilite le formatage des donnees lues
			stoken.eolIsSignificant(false);
		}
		catch (FileNotFoundException e){
			System.out.println("Erreur fatale: Le fichier " + nom + " a disparu !");
			// Cela ne devrait jamais arriver car on verifie l'existance du fichier avant
			// creation de l'objet Lecture
			System.exit(0);
		}
		catch (Exception e) {
			System.out.println("erreur ouverture fichier");
			e.printStackTrace();
			System.exit(0);
		}
    } 

    // Lit le fichier et stocke ce qui est lu dans un Vector
    // Retourne le plus petit id libre
    public int read (Vector cat, Hashtable miniCat) throws IOException, CorruptedFileException {
		boolean exist = false;
		Fiche fic, tmp;
		Vector vect;
		int id = premierIdLibre(); // La 1ere ligne est differente des autres
		while (!isEOF()) {
			fic = ficheSuivante();
			// Insertion de la fixbIche lue ds le catalogue
			cat.addElement(fic);
			// MAJ de miniCat qui permet de retrouver rapidement un id a partir d'un nom
			vect = (Vector)miniCat.get(fic.nom);
			if (vect == null) {
				vect = new Vector();
				vect.addElement(fic);
				miniCat.put(fic.nom,vect);
			}
			else {
				for (int i=0 ; (i<vect.size()) && !exist ; i++) {
					tmp = (Fiche) vect.elementAt(i);
					if (tmp.prenom.equals(fic.prenom))
						exist = true;
				}
				if (!exist)
					vect.addElement(fic);
				exist = false;
			}
		}
		return id;
    }

    // Lit le fichier et stocke ce qui est lu dans un arbre 2-3-4
    // Retourne le plus petit id libre
    public int read (Arbre234 tree, Hashtable miniCat) throws IOException,CorruptedFileException {
		boolean exist = false;
		Fiche fic, tmp;
		Vector vect;
		int id = premierIdLibre(); // La 1ere ligne est differente des autres
		while (!isEOF()) {
			fic = ficheSuivante();
			// Insertion de la fiche lue ds le catalogue
			tree.add(fic);
			// MAJ de miniCat qui permet de retrouver rapidement un id a partir d'un nom
			vect = (Vector)miniCat.get(fic.nom);
			if (vect == null) {
				vect = new Vector();
				vect.addElement(fic);
				miniCat.put(fic.nom,vect);
			}
			else {
				for (int i=0 ; (i<vect.size()) && !exist ; i++) {
					tmp = (Fiche) vect.elementAt(i);
					if (tmp.prenom.equals(fic.prenom))
						exist = true;
				}
				if (!exist)
					vect.addElement(fic);
				exist = false;
			}
		}
		return id;
    }    

    // Lit le fichier et stocke ce qui est lu dans une table de hachage
    // Retourne le plus petit id libre
    public int read (Hashtable hash,Hashtable miniCat) throws IOException,CorruptedFileException {
		boolean exist = false;
		Fiche fic, tmp;
		Vector vect;
		Vector vectId;
		int id = premierIdLibre(); // La 1ere ligne est differente des autres
		while (!isEOF()) {
			fic = ficheSuivante();
			// Insertion de la fiche lue ds le catalogue
			vectId = (Vector) hash.get(new Integer(fic.id));
			if (vectId == null) { //L'id est introduit pour la 1ere fois dans le catalogue
				vectId = new Vector();
				vectId.addElement(fic);
				hash.put(new Integer(fic.id), vectId);
			}
			else { // Le client existe deja ds le catalogue
				vectId.addElement(fic);
			}
			// MAJ de miniCat qui permet de retrouver rapidement un id a partir d'un nom
			vect = (Vector)miniCat.get(fic.nom);
			if (vect == null) {
				vect = new Vector();
				vect.addElement(fic);
				miniCat.put(fic.nom,vect);
			}
			else {
				for (int i=0 ; (i<vect.size()) && !exist ; i++) {
					tmp = (Fiche) vect.elementAt(i);
					if (tmp.prenom.equals(fic.prenom))
						exist = true;
				}
				if (!exist)
					vect.addElement(fic);
				exist = false;
			}
		}
		return id;
    }    

    // Lit la 1ere ligne du fichier catalogue et retourne le 1er num de client libre
    protected int premierIdLibre () throws IOException, CorruptedFileException {
		int id;
		// Sur la 1ere ligne on trouve le 1er num de client non attribue
		stoken.nextToken();
		if (stoken.ttype == StreamTokenizer.TT_NUMBER)
			id = (int) stoken.nval;
		else {
			/* Si on ne trouve pas un entier et uniquement un entier sur la 1ere ligne
			   alors le format du fichier n'est pas valide */
			throw new CorruptedFileException();
		}
		stoken.nextToken(); // Prepare la lecture de la ligne suivante
		return id;
    }

    // Lit une ligne du fichier catalogue en verifiant sa structure et retourne une Fiche
    // Ne peut etre utilise qu'apres premiereFiche() ou ficheSuivante()
    protected Fiche ficheSuivante () throws IOException, CorruptedFileException {
		Date date = new Date(0, 0, 0, 0, 0, 0);
		Fiche entree = new Fiche();
		// Champ num de client
		if (stoken.ttype == StreamTokenizer.TT_NUMBER)
			entree.id = (int) stoken.nval;
		else 
			throw new CorruptedFileException ("num de client");
		// Champ nom de client
		stoken.nextToken();
		if (stoken.ttype == StreamTokenizer.TT_WORD || stoken.ttype == '"')
			entree.nom = stoken.sval;
		else 
			throw new CorruptedFileException ("nom de client");
		// Champ prenom de client
		stoken.nextToken();
		if (stoken.ttype == StreamTokenizer.TT_WORD || stoken.ttype == '"')
			entree.prenom = stoken.sval;
		else 
			throw new CorruptedFileException ("prenom de client");
		// Jour de l'achat
		stoken.nextToken();
		if (stoken.ttype == StreamTokenizer.TT_NUMBER)
			date.setDate((int) stoken.nval);
		else 
			throw new CorruptedFileException ("Jour de l'achat");
		// Mois de l'achat
		stoken.nextToken();
		if (stoken.ttype == StreamTokenizer.TT_NUMBER)
			date.setMonth(((int) stoken.nval)-1);
		else 
			throw new CorruptedFileException ("Mois de l'achat");
		// Annee de l'achat
		stoken.nextToken();
		if (stoken.ttype == StreamTokenizer.TT_NUMBER)
			date.setYear(((int) stoken.nval)-1900);
		else 
			throw new CorruptedFileException ("Annee de l'achat");
		entree.date = date;
		// Champ nom de la voiture
		stoken.nextToken();
		if (stoken.ttype == StreamTokenizer.TT_WORD || stoken.ttype == '"')
			entree.voiture = stoken.sval;
		else 
			throw new CorruptedFileException ("nom de la voiture");
		// Prix
		stoken.nextToken();
		if (stoken.ttype == StreamTokenizer.TT_NUMBER)
			entree.prix = stoken.nval;
		else 
			throw new CorruptedFileException ("Prix");
		stoken.nextToken();
		return entree;
    }

    // Teste si la marque lue est du type EOF
    protected boolean isEOF () {
		return (stoken.ttype == StreamTokenizer.TT_EOF);
    }

    // Ferme le Bufferreader
    public void ferme() throws IOException{
		flot.close();
    }
}