import javax.swing.table.*;

/***********************************************************************
  Cette classe permet de mettre en forme l'affichage des fiches client
  dans l'interface graphique.
  Elle implemente les methodes de la classe abstraite AbstractTableModel
  Seule la methode 'remove' n'est pas une methode de cette classe.
************************************************************************/

class Listing extends AbstractTableModel {
    final String[] columnNames = {" ",
				  " ", 
				  "id",
				  "Nom",
				  "Prenom",
				  "Date",
				  "Voiture",
				  "Prix (euro)"};
    Object[][] data = {{new Boolean(false),
			new Integer(0),
			new Integer(-1),
			" "," "," "," ",
			new Double(0)}};

    public Listing () {
    }

    public Listing (Object[][] tab) {
		data = tab;
    }

	// Retourne le nombre de colonnes dans le tableau
    public int getColumnCount () {
		return columnNames.length;
    }
    
	// Retourne le nombre de lignes dans le tableau
    public int getRowCount () {
		return data.length;
    }
    
	// Retourne le nom de la colonne d'indice 'col'
    public String getColumnName (int col) {
		return columnNames[col];
    }
    
	// Retourne la valeur de la cellule situee ligne 'row' et colonne 'col'
    public Object getValueAt (int row, int col) {
		return data[row][col];
    }
    
    // Permet de formater chaque cellule selon son type
    public Class getColumnClass (int c) {
		return getValueAt(0, c).getClass();
    }
    
    // Seule la 1ere colonne est modifiable. En effet c'est
    // elle qui indique quelles fiches doivent etre modifiees/supprimees
    public boolean isCellEditable (int row, int col) {
		if (col > 0) { 
			return false;
		} 
		else {
			return true;
		}
    }
    
    public void setValueAt (Object value, int row, int col) {
		data[row][col] = value;
		fireTableCellUpdated(row, col);	
    }

    // Afin de supprimer une ligne dans la liste des fiches affichees, on retourne un 
    // nouveau Listing dans lequel cette ligne a ete supprimee.
    public Listing remove (int indice) {
		Object[][] newData = new Object[getRowCount()-1][getColumnCount()];
		int delta = 0;
		for (int i=0; i<getRowCount(); i++) {
			if (i != indice) {
				for (int j=0; j<getColumnCount(); j++) {
					newData[i-delta][j] = data[i][j];
				}
			}
			else {
				delta = 1;
			}
		}
		return new Listing(newData);
    }
}
