import javax.swing.*;
import javax.swing.text.*; 
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*******************************************************************************
  Cette classe est une boite de dialogue destinee a saisir les informations pour
  creer une nouvelle fiche.
  Le type de ces informations est verifie avant de fermer la boite et d'inserer
  cette nouvelle fiche ds le catalogue.
********************************************************************************/

class NewFicheDialog extends JDialog implements ActionListener {
    protected JTextField nomField;
    protected JTextField prenomField;
    protected JTextField jourField;
    protected JTextField moisField;
    protected JTextField anneeField;
    protected JTextField voitureField;
    protected JTextField prixField;
    protected JLabel introTxt;
    protected Fiche fic;

	private NewFicheDialog () {} // Constructeur interdit

	/* Met en place la boite de dialogue en laissant les champs de saisie vides
	   'own' indique la fenetre mere, 'modal' indique si la boite doit etre ou non
	   modale et fid est la fiche a remplir */
    public NewFicheDialog (Fiche fic, Frame own, boolean modal) {
		super(own, modal);
		this.fic = fic;

		Container contentPane = getContentPane();

		// Mise en place du texte d'introduction
		introTxt = new JLabel("Remplissez les champs suivants, puis validez.");
		JPanel introPane = new JPanel();
		introPane.setLayout(new BorderLayout());
		introPane.setBorder(BorderFactory.createEmptyBorder(10,10,0,10));
		introPane.add(introTxt,BorderLayout.CENTER);
		contentPane.add(introPane, BorderLayout.NORTH);

		// Mise en place des boutons de validation/annulation
		JButton ok = new JButton("Valider");
		ok.setActionCommand("Valider");
		ok.addActionListener(this);
		JButton cancel = new JButton("Annuler");
		cancel.setActionCommand("Annuler");
		cancel.addActionListener(this);

		JPanel optionPane = new JPanel();
		optionPane.setLayout(new BoxLayout(optionPane,BoxLayout.X_AXIS));
		optionPane.setBorder(BorderFactory.createEmptyBorder(5,10,10,10));
		optionPane.add(Box.createHorizontalGlue());
		optionPane.add(ok);
		optionPane.add(Box.createRigidArea(new Dimension(10,0)));
		optionPane.add(cancel);
		contentPane.add(optionPane, BorderLayout.SOUTH);

		//Mise en place des JLabel
		JLabel nomTxt = new JLabel("Nom :");
		JLabel prenomTxt = new JLabel("Prenom :");
		JLabel dateTxt = new JLabel("Date :");
		JLabel voitureTxt = new JLabel("Voiture :");
		JLabel prixTxt = new JLabel("Prix (euros) :");

        JPanel labelPane = new JPanel();
        labelPane.setLayout(new GridLayout(0, 1));
		labelPane.setBorder(BorderFactory.createEmptyBorder(10,10,5,5));
		labelPane.add(nomTxt);
		labelPane.add(prenomTxt);
		labelPane.add(dateTxt);
		labelPane.add(voitureTxt);
		labelPane.add(prixTxt);
		contentPane.add(labelPane, BorderLayout.WEST);

		// Mise en place des JTextField
		nomField = new JTextField(20);
        prenomField = new JTextField(20);
        jourField = new JTextField(2);
        moisField = new JTextField(2);
        anneeField = new JTextField(4);
        voitureField = new JTextField(15);
        prixField = new JTextField(7);

		JPanel datePane = new JPanel();
		datePane.setLayout(new FlowLayout(FlowLayout.LEFT,1,0));
		datePane.add(jourField);
		datePane.add(new JLabel("/"));
		datePane.add(moisField);
		datePane.add(new JLabel("/"));
		datePane.add(anneeField);
		JPanel fieldPane = new JPanel();
		fieldPane.setLayout(new BoxLayout(fieldPane,BoxLayout.Y_AXIS));
		fieldPane.setBorder(BorderFactory.createEmptyBorder(10,5,5,10));
		fieldPane.add(nomField);
		fieldPane.add(prenomField);
		fieldPane.add(datePane);
		fieldPane.add(voitureField);
		fieldPane.add(prixField);
        contentPane.add(fieldPane, BorderLayout.CENTER);
    }

    public void actionPerformed (ActionEvent e) {
		if (e.getActionCommand().equals("Annuler")) {
			// On ferme la boite sans modifier la fiche 'fic' passee en param
			this.hide();
			this.dispose();
		}
		else if (e.getActionCommand().equals("Valider")) {
			// On remplit la fiche passee en parametre avec les champs JTextField
			// Tous les champs doivent etre remplis
			if ((nomField.getText().length() == 0) ||
				(prenomField.getText().length() == 0) ||
				(jourField.getText().length() == 0) ||
				(moisField.getText().length() == 0) ||
				(anneeField.getText().length() == 0) ||
				(voitureField.getText().length() == 0) ||
				(prixField.getText().length() == 0) ) {
				String txt = "Tous les champs doivent etre remplis";
				JOptionPane.showMessageDialog(this,
								  txt,
								  "Fiche incomplete",
								  JOptionPane.ERROR_MESSAGE);
			}
			else { 
				try {
					int jour, mois, an;
					jour = (new Integer(jourField.getText())).intValue();
					mois = (new Integer(moisField.getText())).intValue();
					an = (new Integer(anneeField.getText())).intValue();
					Date date = new Date(an-1900, mois-1, jour, 0, 0, 0);
					// on teste si cette date est correcte
					if (an < 0) { // Date non geree par le type Date
						String txt = "Les dates avant JC ne sont pas gerees.";
						JOptionPane.showMessageDialog(this,
										  txt,
										  "Date incompatible",
										  JOptionPane.ERROR_MESSAGE);
					}
					else if (date.getDate()!=jour ||
							 date.getMonth()!=(mois-1)) { // Date inexistante
						String txt = "Cette date n'existe pas.";
						JOptionPane.showMessageDialog(this,
										  txt,
										  "Date incompatible",
										  JOptionPane.ERROR_MESSAGE);
					}
					else { //Tous les champs sont remplis et correctes => on remplit la fiche
						fic.date = date;
						fic.prix = (new Double(prixField.getText())).doubleValue();
						fic.prenom = prenomField.getText();
						fic.voiture = voitureField.getText();
						fic.nom = nomField.getText();
						// On ferme la boite
						this.hide();
						this.dispose();
					}
				}
				catch (NumberFormatException exception) {
					String txt = "Les champs de la date ne doivent contenir que des entiers."
					+ "\nEt celui du prix, un reel";
					JOptionPane.showMessageDialog(this,
								  txt,
								  "Champs incorrectes",
								  JOptionPane.ERROR_MESSAGE);
				}
			}
		}
    }
}